﻿// ----- DisplaySystem Object
// ----- Code by tedGo, includes partial codes by super-gau, fbuser, Br3tt and T.P Wang

function DisplaySystem() {
	var t_rem = window.GetProperty("Remain Time on", false);
	var v_timer = null;
	var v_change = false;

	this.initPos = function(x, y, w, h) {
		this.x_ = x;
		this.y_ = y;
		this.w_ = w;
		this.h_ = h ? h : w / 20 * 3;

		var dlp_ = window.GetProperty("DP3: Display pad left (in pixels)", 0);
		var dtp_ = window.GetProperty("DP4: Display pad top (in pixels)", 0);
		var drp_ = window.GetProperty("DP5: Display pad right (in pixels)", 0);
		var dbp_ = window.GetProperty("DP6: Display pad bottom (in pixels)", 0);

		this.left = this.x_ + Math.abs(dlp_);
		this.top = this.y_ + Math.abs(dtp_);
		this.w = this.w_ - Math.abs(dlp_) - Math.abs(drp_);
		this.h = this.h_ - Math.abs(dtp_) - Math.abs(dbp_);
		this.right = this.left + this.w;
		this.bottom = this.top + this.h;

		this.pixelW = this.w / 400;
		this.pixelH = this.h / 60;

		this.box_w = this.w / 6;
		this.box_h = this.h / 6;
		this.ind_y = this.top + this.h / 10 * 3;
		this.inf_y = this.top + this.h / 5 * 2;
		this.inf_h = this.h / 15 * 8;

		this.ind_font_size = this.pixelW * 7;
		this.inf_font_size = this.pixelW * 29;

		this.signs_left = this.left + Math.floor(this.w - this.pixelW * 18);
		this.time_left = this.left + this.pixelW * 164;
	}

	this.traceMouse = function(x, y) {
		return x >= this.left && x <= this.right && y >= this.top && y <= this.bottom;
	}

	this.InitColours = function() {
		var dbc_tmp = window.GetProperty("DP1: Display background (R-G-B-A)", "31-31-31-255");
		var colr_arr = [RGB(41, 143, 204), RGB(6, 176, 37), RGB(191, 223, 255), RGB(255, 0, 0), RGB(255, 255, 255), RGB(255, 223, 63)];
		display_back_colour = CustomColour(dbc_tmp);
		disp_arr = ["Blue", "Green", "Light Blue", "Red", "White", "Yellow"];
		disp_colour = window.GetProperty("DP2: Display Colour (0-5)", 0);
		active_colour = colr_arr[disp_colour];
		inactive_colour = combColours(display_back_colour, colr_arr[4], 0.02);
	}

	this.setColours = function() {
		var metadb = fb.GetNowPlaying();

		if (metadb) {
			var tmp = ["$if($stricmp(%__encoding%,lossless),1)", "$if($not($stricmp(%__encoding%,lossless)),1)", "$if($and($greater(%samplerate%,88199),$greater(%__bitspersample%,23)),1)", "$if($greater(%__channels%,2),1)", "[%__md5%]", "[%replaygain_track_gain%]"];
			for (var i = 0; i >= 0 && i <= 5; i++) this.Colours[i] = fb.TitleFormat(tmp[i]).Eval() ? active_colour : inactive_colour;
			this.Colours[6] = fb.TitleFormat("[%tracknumber%]").Eval() ? ui_btntxtcol : inactive_colour;
			this.Colours[7] = fb.TitleFormat("[%totaltracks%]").Eval() ? ui_btntxtcol : inactive_colour;
			this.Colours[8] = ui_btntxtcol;
		} else for (var i = 0; i >= 0 && i <= 8; i++) this.Colours[i] = inactive_colour;
	}

	this.setTrackNo = function() {
		var metadb = fb.GetNowPlaying();

		if (metadb) {
			this.TrackNo = fb.TitleFormat("[$num(%tracknumber%,2)]").Eval();
			this.TotalNo = fb.TitleFormat("[$num(%totaltracks%,2)]").Eval();
			this.Trackinfo = fb.PlaybackLength <= 0 ? false : true;
		} else this.TrackNo = this.TotalNo = "";
	}

	this.setPBTime = function() {
		this.Elapse = TimeFmt(fb.PlaybackTime);
		this.Remain = fb.PlaybackLength < 0 ? "-- : -- : --" : TimeFmt(fb.PlaybackLength - fb.PlaybackTime);
	}

	this.setBitrate = function() {
		this.Bitrate = fb.TitleFormat("%bitrate%").Eval();
	}

	this.init = function() {
		this.setColours();
		this.setTrackNo();
		this.setPBTime();
		this.setBitrate();
	}

	this.repaintALL = function() {window.RepaintRect(this.left, this.top, this.w, this.h)};
	this.repaintSAC = function() {window.RepaintRect(this.signs_left, this.ind_y + 2, this.pixelW * 18 + 2, this.pixelW * 12 + 2)};
	this.repaintPBO = function() {window.RepaintRect(this.signs_left, this.top + Math.floor(this.pixelH * 39), this.pixelW * 18 + 2, this.pixelW * 12 + 2 )};
	this.repaintPBT = function() {!v_change && window.RepaintRect(this.time_left, this.inf_y, this.pixelW * 112, this.inf_h)};
	this.repaintVOL = function() {window.RepaintRect(this.time_left, this.ind_y, this.pixelW * 210, this.inf_h + this.box_h)};

	this.draw = function(gr) {
		!window.IsTransparent && gr.FillSolidRect(this.x_, this.y_, this.w_, this.h_, display_back_colour);

		gr.SetInterpolationMode(7);
		var ind_font = gdi.Font("Arial Black", this.ind_font_size);
		var inf_font = gdi.Font("Microsoft Sans Serif", this.inf_font_size);

		var ind_arr = ["LOSSLESS", "LOSSY", "HI-RES", "MULTI-CH", "AUDIO MD5", "REPLAYGAIN"];
		for (var i = 0; i >= 0 && i <= 5; i++) {
			gr.DrawRect(this.left + this.box_w * i + 1, this.top, this.box_w - 2, this.box_h, this.pixelW, this.Colours[i]);
			gr.GdiDrawText(ind_arr[i], ind_font, this.Colours[i], this.left + this.box_w * i + 1, this.top, this.box_w - 2, this.box_h, 5);
		}

		var inf_arr = ["TRACK", "TOTAL", "ELAPSED", "TIME", "REMAINING", "VOLUME", "KBPS"];
		var tmp_c = [this.Colours[6], this.Colours[7], v_change || t_rem ? inactive_colour : this.Colours[8], v_change ? inactive_colour : this.Colours[8], v_change || !t_rem ? inactive_colour : this.Colours[8], v_change ? ui_btntxtcol : inactive_colour, v_change ? inactive_colour : this.Colours[8]];
		var tmp_x = [0, this.pixelW * 82, this.pixelW * 165, this.pixelW * 205, this.pixelW * 230, this.pixelW * 312, this.pixelW * 352];
		var tmp_w = []; for (var j in inf_arr) tmp_w.push(gr.CalcTextWidth(inf_arr[j], ind_font));
		for (var k = 0; k >= 0 && k <= 6; k++) gr.GdiDrawText(inf_arr[k], ind_font, tmp_c[k], this.left + tmp_x[k], this.ind_y, tmp_w[k], this.box_h, 0);

		if (this.Trackinfo) {
			gr.GdiDrawText(this.TrackNo, inf_font, active_colour, this.left, this.inf_y, this.pixelW * 82, this.inf_h, 0);
			gr.GdiDrawText(this.TotalNo, inf_font, active_colour, this.left + this.pixelW * 82, this.inf_y, this.pixelW * 82, this.inf_h, 0);
		} else fb.IsPlaying && gr.GdiDrawText("Stream", inf_font, active_colour, this.left, this.inf_y, this.pixelW * 164, this.inf_h, 0);

		if (v_change) gr.GdiDrawText(fb.Volume.toFixed(2) + " dB", inf_font, active_colour, this.time_left, this.inf_y, this.pixelW * 210, this.inf_h, 2);
		else {
			if (fb.IsPlaying) {
				var t_fmt = fb.IsPaused ? "-Paused-" : t_rem ? this.Remain : this.Elapse;
				gr.GdiDrawText(t_fmt, inf_font, active_colour, this.time_left, this.inf_y, this.pixelW * 112, this.inf_h, 1);
				gr.GdiDrawText(this.Bitrate, inf_font, active_colour, this.left + this.pixelW * 294, this.inf_y, this.w / 5, this.inf_h, 2);

			}
		}

		g_signs && gr.DrawImage(g_signs, this.signs_left, this.ind_y + 2, this.pixelW * 18, this.pixelW * 12, 1944, fb.StopAfterCurrent ? disp_colour * 66 : 264, 54, 36, 0, fb.StopAfterCurrent ? 255 : 5);
		g_signs && gr.DrawImage(g_signs, this.signs_left, this.top + this.pixelH * 39, this.pixelW * 18, this.pixelW * 12, 2004 + plman.PlaybackOrder * 60, fb.IsPlaying ? disp_colour * 66 : 264, 54, 36, 0, fb.IsPlaying ? 255 : 5);
	}

	this.MouseRbtnUp = function(x, y) {
		var a = window.CreatePopupMenu();
		var idx;

		for (var i = 0; i < disp_arr.length; i++) a.AppendMenuItem(0, 1 + i, disp_arr[i]);
		a.CheckMenuRadioItem(1, 6, disp_colour + 1);
		a.AppendMenuSeparator();
		a.AppendMenuItem(0, 7, "Display properties");

		idx = a.TrackPopupMenu(x, y);
		if (idx >= 1 && idx <= 6) {
			window.SetProperty("DP2: Display Colour (0-5)", idx - 1);
			this.InitColours();
			this.setColours();
			this.repaintALL();
			fb.RunMainMenuCommand("DarkOne4Mod - " + disp_arr[disp_colour]);
		}
		idx == 7 && window.ShowProperties();

		a.Dispose();
	}

	this.VolumeChange = function(val) {
		if (v_timer) {
			window.ClearTimeout(v_timer);
			v_timer = null;
		}

		v_timer = window.SetTimeout(
			function () {
				display_system.repaintVOL();
				window.ClearTimeout(v_timer);
				v_timer = null;
				v_change = false;
		}, 3000);

		v_change = true;
		this.repaintVOL();
	}

	this.NotifyData = function(name, info) {
		if (name == "remTime") {
			window.SetProperty("Remain Time on", info == true ? true : false);
			t_rem = window.GetProperty("Remain Time on");
			fb.IsPlaying && !v_change && window.RepaintRect(this.time_left, this.ind_y, this.pixelW * 120, this.inf_h + this.box_h);
		}
	}

	this.PlayNewTrack = function(metadb) {
		this.init();
		this.repaintALL();
	}

	this.PlayTime = function(time) {
		this.setPBTime();
		this.repaintPBT();
	}

	this.PlayDynInfo = function() {
		this.setBitrate()
		!v_change && window.RepaintRect(this.left + this.pixelW * 294, this.inf_y, this.w / 5, this.inf_h);
	}

	this.PlayEdited = function() {
		this.init();
		this.repaintALL();
	}

	this.onStop = function(reason) {
		if (reason != 2) {
			this.init();
			fb.RunMainMenuCommand("ELPlaylist/Redraw");
		}

		this.repaintALL();
	}

	this.InitColours();
	this.Colours = [];
	this.init();
}

display_system = new DisplaySystem();