﻿// ----- Volumeknob Object
// ----- Code by tedGo, based on a sample created by T.P. Wang, using partial code by Br3tt

var vknobOptions = {};
var v_change = false;
var v_drag = false;
var v_timer = null;

function VolumeKnob(options) {
	this.line_width = options && options.line_width ? options.line_width : 0;
	this.line_normal = options && options.line_normal ? options.line_normal : null;
	this.line_hover = options && options.line_hover ? options.line_hover : this.line_normal;
	this.colour_l = this.line_normal;
	this.back_normal = options && options.back_normal ? options.back_normal : null;
	this.back_hover = options && options.back_hover ? options.back_hover : this.back_normal;
	this.colour_b = this.back_normal;
	this.hover = false;

	this.initPos = function(x, y, w, h) {
		this.left = x;
		this.top = y;
		this.w = w;
		this.h = h;
		this.right = x + w;
		this.bottom = y + h;
	}

	this.traceMouse = function(x, y) {
		return x >= this.left && x <= this.right && y >= this.top && y <= this.bottom;
	}

	this.calc_theta = function(x, y) {
		x -= this.left + this.w / 2;
		y -= this.top + this.h / 2;
		theta = Math.atan2(y, x) / Math.PI * 180;
		if (theta <= 90 && theta > 45) return 270;
		if (theta > 90 && theta < 135) return 0;
		return theta >= 0 ? theta > 90 ? theta - 135 : theta + 225 : theta + 225
	}

	this.draw = function(gr) {
		var g_R = Math.min(this.w, this.h) / Math.PI;

		this.colour_b && gr.FillEllipse(this.left, this.top, this.w, this.h, this.colour_b);
		this.colour_l && this.line_width > 0 && gr.DrawEllipse(this.left, this.top, this.w, this.h, this.line_width, this.colour_l);
		gr.DrawLine(this.left + 1, this.bottom - 1, this.left + 1 + this.w / 12, this.bottom - 1 - this.h / 12, 2, ui_btntxtcol);
		gr.DrawLine(this.right - 1 - this.w / 12, this.bottom - 1 - this.w / 12, this.right - 1, this.bottom - 1, 2, ui_btntxtcol);

		theta = (Math.pow(10, fb.Volume / 50) - 0.001) / 0.999 * 270;
		posA = (theta - 45) * Math.PI / 180;
		cosinusA = Math.cos(posA);
		sinusA = Math.sin(posA);
		posX = (this.left + (this.w / 2) - ww / 200) - (cosinusA * g_R);
		posY = (this.top + (this.h / 2) - ww / 200) - (sinusA * g_R);

		x_col = fb.Volume == -100 ? RGBA(255, 0, 0, 255) : v_change ? RGBA(41, 143, 204, 255) : RGBA(0, 0, 0, 255);
		gr.FillEllipse(posX , posY, ww / 100, ww / 100, x_col);
	}

	this.Repaint = function() {
		window.RepaintRect(this.left, this.top, this.w, this.h);
	}

	this.on_mouse_move = function(x, y) {
		if (this.traceMouse(x, y)) {
			if (v_drag) {
				n = this.calc_theta(x, y) / 270;
				v = 50 * Math.log(0.99 * n + 0.01) / Math.LN10;
				if (fb.Volume != v) fb.Volume = v;
			}
			this.colour_b = this.back_hover;
			this.colour_l = this.line_hover;
			!this.hover && this.Repaint();
			this.hover = true;
		} else {
			this.colour_b = this.back_normal;
			this.colour_l = this.line_normal;
			this.hover && this.Repaint();
			this.hover = false;
		}

		this.mouseX = x;
		this.mouseY = y;
	}

	this.on_mouse_lbtn_down = function(x, y) {
		v_drag = true;
	}

	this.on_mouse_lbtn_up = function(x, y) {
		this.on_mouse_move(x,y);
		v_drag = false;
	}

	this.on_mouse_wheel = function(step) {
		if (this.traceMouse(this.mouseX, this.mouseY)) step > 0 ? fb.VolumeUp() : fb.VolumeDown();
	}

	this.on_mouse_rbtn_up = function(x, y) {
		var m = window.CreatePopupMenu();
		var q;

		m.AppendMenuItem(fb.Volume == 0 ? 1 : 0, 1, "Up");
		m.AppendMenuItem(fb.Volume == -100 ? 1 : 0, 2, "Down");
		m.AppendMenuItem(fb.Volume == -100 ? 8 : 0, 3, "Volume Mute");
		m.AppendMenuSeparator();
		for (var i = 0; i >= 0 && i <= 7; i++) m.AppendMenuItem(fb.Volume == i * -3 ? 8 : 0, i + 4, "Set to " + (i * -3) + " db");

		q = m.TrackPopupMenu(x, y);

		switch (true) {
			case (q == 1):
				fb.VolumeUp();
				break;

			case (q == 2):
				fb.VolumeDown();
				break;

			case (q == 3):
				fb.VolumeMute();
				break;

			case (q >= 4 && q <= 11):
				fb.RunMainMenuCommand("Set to -" + (q - 4) * 3 + " db");
				this.colour_b = this.back_normal;
				break;
		}

		m.Dispose();
	}
}

function on_volume_change(val) {
	if (v_timer) {
		window.ClearTimeout(v_timer);
		v_timer = null;
	}

	v_timer = window.SetTimeout(
		function () {
			volknob.Repaint();
			window.ClearTimeout(v_timer);
			v_timer = null;
			v_change = false;
		}, 3000);

	v_change = true;
	volknob.Repaint();
}